94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
149 lines
3.9 KiB
Python
149 lines
3.9 KiB
Python
# SPDX-License-Identifier: Apache-2.0
|
|
"""Fused Gemma RMSNorm Triton kernels for MiniMax-M3 on AMD ROCm.
|
|
|
|
Gemma RMSNorm = ``normalize(x) * (1 + weight)``, computed in a single fp32 pass.
|
|
On ROCm with AITER, ``GemmaRMSNorm.forward_hip`` otherwise falls back to a
|
|
~8-op PyTorch sequence: ``sgl_kernel``'s Gemma kernels are CUDA-only, and
|
|
AITER's ``rmsnorm2d_fwd`` requires weight.dtype == activation.dtype (fp32
|
|
weight + bf16 activation silently corrupts on gfx950). These kernels read
|
|
strided inputs, so they serve both the full-hidden norms and the per-head
|
|
q/k/index norms (non-contiguous ``qkv.split`` views).
|
|
"""
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
@triton.jit
|
|
def _gemma_rmsnorm_kernel(
|
|
x_ptr,
|
|
w_ptr,
|
|
out_ptr,
|
|
n_cols,
|
|
stride_row,
|
|
stride_col,
|
|
eps,
|
|
BLOCK_N: tl.constexpr,
|
|
):
|
|
row = tl.program_id(0)
|
|
cols = tl.arange(0, BLOCK_N)
|
|
mask = cols < n_cols
|
|
x = tl.load(x_ptr + row * stride_row + cols * stride_col, mask=mask, other=0.0).to(
|
|
tl.float32
|
|
)
|
|
var = tl.sum(x * x, axis=0) / n_cols
|
|
rstd = 1.0 / tl.sqrt(var + eps)
|
|
w = tl.load(w_ptr + cols, mask=mask, other=0.0).to(tl.float32)
|
|
out = x * rstd * (1.0 + w)
|
|
tl.store(
|
|
out_ptr + row * n_cols + cols,
|
|
out.to(out_ptr.dtype.element_ty),
|
|
mask=mask,
|
|
)
|
|
|
|
|
|
@triton.jit
|
|
def _gemma_fused_add_rmsnorm_kernel(
|
|
x_ptr,
|
|
res_ptr,
|
|
w_ptr,
|
|
out_ptr,
|
|
res_out_ptr,
|
|
n_cols,
|
|
stride_xrow,
|
|
stride_xcol,
|
|
stride_rrow,
|
|
stride_rcol,
|
|
eps,
|
|
BLOCK_N: tl.constexpr,
|
|
):
|
|
row = tl.program_id(0)
|
|
cols = tl.arange(0, BLOCK_N)
|
|
mask = cols < n_cols
|
|
x = tl.load(
|
|
x_ptr + row * stride_xrow + cols * stride_xcol, mask=mask, other=0.0
|
|
).to(tl.float32)
|
|
r = tl.load(
|
|
res_ptr + row * stride_rrow + cols * stride_rcol, mask=mask, other=0.0
|
|
).to(tl.float32)
|
|
s = x + r
|
|
# residual_out is the pre-norm sum (consumed by the next layer's add).
|
|
tl.store(
|
|
res_out_ptr + row * n_cols + cols,
|
|
s.to(res_out_ptr.dtype.element_ty),
|
|
mask=mask,
|
|
)
|
|
var = tl.sum(s * s, axis=0) / n_cols
|
|
rstd = 1.0 / tl.sqrt(var + eps)
|
|
w = tl.load(w_ptr + cols, mask=mask, other=0.0).to(tl.float32)
|
|
out = s * rstd * (1.0 + w)
|
|
tl.store(
|
|
out_ptr + row * n_cols + cols,
|
|
out.to(out_ptr.dtype.element_ty),
|
|
mask=mask,
|
|
)
|
|
|
|
|
|
def _num_warps(block_n: int) -> int:
|
|
if block_n >= 4096:
|
|
return 16
|
|
if block_n >= 1024:
|
|
return 8
|
|
return 4
|
|
|
|
|
|
def gemma_rmsnorm(x: torch.Tensor, weight: torch.Tensor, eps: float) -> torch.Tensor:
|
|
"""Gemma RMSNorm = normalize(x) * (1 + weight), fp32 math, single pass."""
|
|
orig_shape = x.shape
|
|
n = orig_shape[-1]
|
|
x2 = x.reshape(-1, n)
|
|
m = x2.shape[0]
|
|
out = torch.empty((m, n), dtype=x.dtype, device=x.device)
|
|
block_n = triton.next_power_of_2(n)
|
|
_gemma_rmsnorm_kernel[(m,)](
|
|
x2,
|
|
weight,
|
|
out,
|
|
n,
|
|
x2.stride(0),
|
|
x2.stride(1),
|
|
eps,
|
|
BLOCK_N=block_n,
|
|
num_warps=_num_warps(block_n),
|
|
)
|
|
return out.reshape(orig_shape)
|
|
|
|
|
|
def gemma_fused_add_rmsnorm(
|
|
x: torch.Tensor,
|
|
residual: torch.Tensor,
|
|
weight: torch.Tensor,
|
|
eps: float,
|
|
):
|
|
"""Fused (x + residual) then Gemma RMSNorm; returns (normed, pre-norm sum)."""
|
|
orig_shape = x.shape
|
|
n = orig_shape[-1]
|
|
x2 = x.reshape(-1, n)
|
|
r2 = residual.reshape(-1, n)
|
|
m = x2.shape[0]
|
|
out = torch.empty((m, n), dtype=x.dtype, device=x.device)
|
|
res_out = torch.empty((m, n), dtype=x.dtype, device=x.device)
|
|
block_n = triton.next_power_of_2(n)
|
|
_gemma_fused_add_rmsnorm_kernel[(m,)](
|
|
x2,
|
|
r2,
|
|
weight,
|
|
out,
|
|
res_out,
|
|
n,
|
|
x2.stride(0),
|
|
x2.stride(1),
|
|
r2.stride(0),
|
|
r2.stride(1),
|
|
eps,
|
|
BLOCK_N=block_n,
|
|
num_warps=_num_warps(block_n),
|
|
)
|
|
return out.reshape(orig_shape), res_out.reshape(orig_shape)
|